@@ -32,24 +32,29 @@ r"""
     The graph is inserted as a PNG+image map into HTML and a PDF in
     LaTeX.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 import sys
 import inspect
-import builtins as __builtin__ # as __builtin__ is for lib2to3 compatibility
 try:
     from hashlib import md5
 except ImportError:
     from md5 import md5
 
+from six import text_type
+from six.moves import builtins
+
 from docutils import nodes
 from docutils.parsers.rst import directives
 
+import sphinx
 from sphinx.ext.graphviz import render_dot_html, render_dot_latex, \
     render_dot_texinfo
+from sphinx.pycode import ModuleAnalyzer
+from sphinx.util import force_decode
 from sphinx.util.compat import Directive
 
 
@@ -121,7 +126,7 @@ class InheritanceGraph(object):
             return [todoc]
         elif inspect.ismodule(todoc):
             classes = []
-            for cls in list(todoc.__dict__.values()):
+            for cls in todoc.__dict__.values():
                 if inspect.isclass(cls) and cls.__module__ == todoc.__name__:
                     classes.append(cls)
             return classes
@@ -143,10 +148,10 @@ class InheritanceGraph(object):
         displayed node names.
         """
         all_classes = {}
-        builtins = list(vars(__builtin__).values())
+        py_builtins = vars(builtins).values()
 
         def recurse(cls):
-            if not show_builtins and cls in builtins:
+            if not show_builtins and cls in py_builtins:
                 return
             if not private_bases and cls.__name__.startswith('_'):
                 return
@@ -158,7 +163,10 @@ class InheritanceGraph(object):
             tooltip = None
             try:
                 if cls.__doc__:
+                    enc = ModuleAnalyzer.for_module(cls.__module__).encoding
                     doc = cls.__doc__.strip().split("\n")[0]
+                    if not isinstance(doc, text_type):
+                        doc = force_decode(doc, enc)
                     if doc:
                         tooltip = '"%s"' % doc.replace('"', '\\"')
             except Exception:  # might raise AttributeError for strange classes
@@ -167,7 +175,7 @@ class InheritanceGraph(object):
             baselist = []
             all_classes[cls] = (nodename, fullname, baselist, tooltip)
             for base in cls.__bases__:
-                if not show_builtins and base in builtins:
+                if not show_builtins and base in py_builtins:
                     continue
                 if not private_bases and base.__name__.startswith('_'):
                     continue
@@ -187,7 +195,7 @@ class InheritanceGraph(object):
         completely general.
         """
         module = cls.__module__
-        if module == '__builtin__':
+        if module in ('__builtin__', 'builtins'):
             fullname = cls.__name__
         else:
             fullname = '%s.%s' % (module, cls.__name__)
@@ -219,10 +227,10 @@ class InheritanceGraph(object):
     }
 
     def _format_node_attrs(self, attrs):
-        return ','.join(['%s=%s' % x for x in list(attrs.items())])
+        return ','.join(['%s=%s' % x for x in sorted(attrs.items())])
 
     def _format_graph_attrs(self, attrs):
-        return ''.join(['%s=%s;\n' % x for x in list(attrs.items())])
+        return ''.join(['%s=%s;\n' % x for x in sorted(attrs.items())])
 
     def generate_dot(self, name, urls={}, env=None,
                      graph_attrs={}, node_attrs={}, edge_attrs={}):
@@ -256,6 +264,7 @@ class InheritanceGraph(object):
             this_node_attrs = n_attrs.copy()
             if fullname in urls:
                 this_node_attrs['URL'] = '"%s"' % urls[fullname]
+                this_node_attrs['target'] = '"_top"'
             if tooltip:
                 this_node_attrs['tooltip'] = tooltip
             res.append('  "%s" [%s];\n' %
@@ -303,7 +312,7 @@ class InheritanceDiagram(Directive):
         # Create a graph starting with the list of classes
         try:
             graph = InheritanceGraph(
-                class_names, env.temp_data.get('py:module'),
+                class_names, env.ref_context.get('py:module'),
                 parts=node['parts'],
                 private_bases='private-bases' in self.options)
         except InheritanceException as err:
@@ -340,12 +349,20 @@ def html_visit_inheritance_diagram(self, node):
     name = 'inheritance%s' % graph_hash
 
     # Create a mapping from fully-qualified class names to URLs.
+    graphviz_output_format = self.builder.env.config.graphviz_output_format.upper()
+    current_filename = self.builder.current_docname + self.builder.out_suffix
     urls = {}
     for child in node:
         if child.get('refuri') is not None:
-            urls[child['reftitle']] = child.get('refuri')
+            if graphviz_output_format == 'SVG':
+                urls[child['reftitle']] = "../" + child.get('refuri')
+            else:
+                urls[child['reftitle']] = child.get('refuri')
         elif child.get('refid') is not None:
-            urls[child['reftitle']] = '#' + child.get('refid')
+            if graphviz_output_format == 'SVG':
+                urls[child['reftitle']] = '../' + current_filename + '#' + child.get('refid')
+            else:
+                urls[child['reftitle']] = '#' + child.get('refid')
 
     dotcode = graph.generate_dot(name, urls, env=self.builder.env)
     render_dot_html(self, node, dotcode, [], 'inheritance', 'inheritance',
@@ -400,3 +417,4 @@ def setup(app):
     app.add_config_value('inheritance_graph_attrs', {}, False),
     app.add_config_value('inheritance_node_attrs', {}, False),
     app.add_config_value('inheritance_edge_attrs', {}, False),
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
