@@ -8,16 +8,17 @@
 
     :author: Sebastian Wiesner
     :contact: basti.wiesner@gmx.net
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import sys
 import os
-import types
-from io import StringIO
 from distutils.cmd import Command
-from distutils.errors import DistutilsOptionError
+from distutils.errors import DistutilsOptionError, DistutilsExecError
+
+from six import StringIO, string_types
 
 from sphinx.application import Sphinx
 from sphinx.util.console import darkred, nocolor, color_terminal
@@ -76,10 +77,10 @@ class BuildDoc(Command):
         ('today=', None, 'How to format the current date, used as the '
          'replacement for |today|'),
         ('link-index', 'i', 'Link index.html to the master doc'),
+        ('copyright', None, 'The copyright string'),
     ]
     boolean_options = ['fresh-env', 'all-files', 'link-index']
 
-
     def initialize_options(self):
         self.fresh_env = self.all_files = False
         self.source_dir = self.build_dir = None
@@ -90,6 +91,7 @@ class BuildDoc(Command):
         self.today = ''
         self.config_dir = None
         self.link_index = False
+        self.copyright = ''
 
     def _guess_source_dir(self):
         for guess in ('doc', 'docs'):
@@ -108,7 +110,7 @@ class BuildDoc(Command):
         if val is None:
             setattr(self, option, default)
             return default
-        elif not isinstance(val, str):
+        elif not isinstance(val, string_types):
             raise DistutilsOptionError("'%s' must be a %s (got `%s`)"
                                        % (option, what, val))
         return val
@@ -137,7 +139,6 @@ class BuildDoc(Command):
 
     def run(self):
         if not color_terminal():
-            # Windows' poor cmd box doesn't understand ANSI sequences
             nocolor()
         if not self.verbose:
             status_stream = StringIO()
@@ -145,13 +146,15 @@ class BuildDoc(Command):
             status_stream = sys.stdout
         confoverrides = {}
         if self.project:
-             confoverrides['project'] = self.project
+            confoverrides['project'] = self.project
         if self.version:
-             confoverrides['version'] = self.version
+            confoverrides['version'] = self.version
         if self.release:
-             confoverrides['release'] = self.release
+            confoverrides['release'] = self.release
         if self.today:
-             confoverrides['today'] = self.today
+            confoverrides['today'] = self.today
+        if self.copyright:
+            confoverrides['copyright'] = self.copyright
         app = Sphinx(self.source_dir, self.config_dir,
                      self.builder_target_dir, self.doctree_dir,
                      self.builder, confoverrides, status_stream,
@@ -159,12 +162,15 @@ class BuildDoc(Command):
 
         try:
             app.build(force_all=self.all_files)
+            if app.statuscode:
+                raise DistutilsExecError(
+                    'caused by %s builder.' % app.builder.name)
         except Exception as err:
             from docutils.utils import SystemMessage
             if isinstance(err, SystemMessage):
                 print(darkred('reST markup error:'), file=sys.stderr)
-                print(err.args[0].encode('ascii',
-                                                       'backslashreplace'), file=sys.stderr)
+                print(err.args[0].encode('ascii', 'backslashreplace'),
+                      file=sys.stderr)
             else:
                 raise
 
